'' +--------------------------------------------------------------------------+
'' | Cluso's Propeller Operating System - OS boot program: _BOOT_OS.BIN v1.12 |
'' +--------------------------------------------------------------------------+
'' |  Author(s):     (c)2011 Kwabena W. Agyman (Kye)  "SD-MMC_FATEngine.spin" |
'' |                 (c)2016 "Cluso99" (Ray Rodrick)   www.clusos.com         |
'' |  Modifications:                                                          |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' This program resides in EEPROM and locates/loads/runs a program
''    "BOOTPROP.BIN" from the SD Card in FAT16/32 format.
''    Typically used to boot my Propeller Operating System.
''
'' RR20160128   v000   (similar to my eeprom boot program for RamBlade)
'' RR20160128   v1.00  first release
'' RR20160129   v1.10  add compile options (use homespun.exe + boot.bat to compile)
'' RR20160725   v1.11  display SD volume label, fix typo
'' RR20170117   v1.12  add 6M5RB1 (RamBlade)


DAT
bootfile1     byte      "_BOOT_P1.BIN",0        ' load/run this file from SD card
bootfile2     byte      "_OS.CMD",0             ' load/run this file from SD card


CON

#ifdef custom
'' +--------------------------------------------------------------------------+
'' |            HARDWARE DEFINITIONS (change to suit your board)              |
'' +--------------------------------------------------------------------------+
  _xinfreq = 5_000_000          ' \ 5MHz*16
  _clkmode = xtal1 + pll16x     ' /

  _SIpin        = 31            ' \ usual serial input pin  (from TXD on pc)
  _SOpin        = 30            ' |              output pin (to   RXD on pc)
  _SIOmode      = 0             ' | usual mode
  _SIObaud      = 115200        ' / common baud (serial speed)
  _SIOpins      = _SIOmode << 16 | _SIpin << 8 | _SOpin

  _SDpin_CS     = 3             ' \ SD card pins: CS  (active low)
  _SDpin_DI     = 2             ' |               DI  (to SD)
  _SDpin_CLK    = 1             ' |               CLK (to SD)
  _SDpin_DO     = 0             ' |               DO  (fm SD)
' _SDpin_CD     = -1            ' | card detect   (can be DI)
' _SDpin_WP     = -1            ' / write protect (can be CLK) (not on microSD cards)
  _SDpins       = _SDpin_CS << 24 | _SDpin_DI << 16 | _SDpin_CLK << 8 | _SDpin_DO
'' +--------------------------------------------------------------------------+
#elseifdef 5M0
'' +--------------------------------------------------------------------------+
'' |    HARDWARE DEFINITIONS (5.00MHz x16, SD on P0..P3, Serial on P30/P31)   |
'' +--------------------------------------------------------------------------+
  _xinfreq = 5_000_000          ' \ 5MHz*16
  _clkmode = xtal1 + pll16x     ' /

  _SIpin        = 31            ' \ usual serial input pin  (from TXD on pc)
  _SOpin        = 30            ' |              output pin (to   RXD on pc)
  _SIOmode      = 0             ' | usual mode
  _SIObaud      = 115200        ' / common baud (serial speed)
  _SIOpins      = _SIOmode << 16 | _SIpin << 8 | _SOpin

  _SDpin_CS     = 3             ' \ SD card pins: CS  (active low)
  _SDpin_DI     = 2             ' |               DI  (to SD)
  _SDpin_CLK    = 1             ' |               CLK (to SD)
  _SDpin_DO     = 0             ' |               DO  (fm SD)
' _SDpin_CD     = -1            ' | card detect   (can be DI)
' _SDpin_WP     = -1            ' / write protect (can be CLK) (not on microSD cards)
  _SDpins       = _SDpin_CS << 24 | _SDpin_DI << 16 | _SDpin_CLK << 8 | _SDpin_DO
'' +--------------------------------------------------------------------------+
#elseifdef 10M0
'' +--------------------------------------------------------------------------+
'' |    HARDWARE DEFINITIONS (10.00MHz x8, SD on P0..P3, Serial on P30/P31)   |
'' +--------------------------------------------------------------------------+
  _xinfreq = 10_000_000         ' \ 10MHz*8
  _clkmode = xtal1 + pll8x      ' /

  _SIpin        = 31            ' \ usual serial input pin  (from TXD on pc)
  _SOpin        = 30            ' |              output pin (to   RXD on pc)
  _SIOmode      = 0             ' | usual mode
  _SIObaud      = 115200        ' / common baud (serial speed)
  _SIOpins      = _SIOmode << 16 | _SIpin << 8 | _SOpin

  _SDpin_CS     = 3             ' \ SD card pins: CS  (active low)
  _SDpin_DI     = 2             ' |               DI  (to SD)
  _SDpin_CLK    = 1             ' |               CLK (to SD)
  _SDpin_DO     = 0             ' |               DO  (fm SD)
' _SDpin_CD     = -1            ' | card detect   (can be DI)
' _SDpin_WP     = -1            ' / write protect (can be CLK) (not on microSD cards)
  _SDpins       = _SDpin_CS << 24 | _SDpin_DI << 16 | _SDpin_CLK << 8 | _SDpin_DO
'' +--------------------------------------------------------------------------+
#elseifdef 12M12
'' +--------------------------------------------------------------------------+
'' |    HARDWARE DEFINITIONS: P8XBlade2/CpuBlade7 (12MHz x8, P12-15, P30/P31) |
'' +--------------------------------------------------------------------------+
  _xinfreq = 12_000_000         ' \ P8XBlade2/CpuBlade2 12MHz*8
  _clkmode = xtal1 + pll8x      ' /

  _SIpin        = 31            ' \ usual serial input pin  (from TXD on pc)
  _SOpin        = 30            ' |              output pin (to   RXD on pc)
  _SIOmode      = 0             ' | usual mode
  _SIObaud      = 115200        ' / common baud (serial speed)
  _SIOpins      = _SIOmode << 16 | _SIpin << 8 | _SOpin

  _SDpin_CS     = 15            ' \ SD card pins: CS  (active low)
  _SDpin_DI     = 14            ' |               DI  (to SD)
  _SDpin_CLK    = 13            ' |               CLK (to SD)
  _SDpin_DO     = 12            ' |               DO  (fm SD)
' _SDpin_CD     = -1            ' | card detect   (can be DI)
' _SDpin_WP     = -1            ' / write protect (can be CLK) (not on microSD cards)
  _SDpins       = _SDpin_CS << 24 | _SDpin_DI << 16 | _SDpin_CLK << 8 | _SDpin_DO
'' +--------------------------------------------------------------------------+
#elseifdef 6M5RB1
'' +--------------------------------------------------------------------------+
'' |    HARDWARE DEFINITIONS: RamBlade-1 (6.5MHz x16, P24/26/25/19, P22/P23)  |
'' +--------------------------------------------------------------------------+
  _xinfreq = 6_500_000          ' \ RamBlade-1 6.5MHz*16
  _clkmode = xtal1 + pll16x     ' /

  _SIpin        = 23            ' \ usual serial input pin  (from TXD on pc)
  _SOpin        = 22            ' |              output pin (to   RXD on pc)
  _SIOmode      = 0             ' | usual mode
  _SIObaud      = 115200        ' / common baud (serial speed)
  _SIOpins      = _SIOmode << 16 | _SIpin << 8 | _SOpin

  _SDpin_CS     = 19            ' \ SD card pins: CS  (active low)
  _SDpin_DI     = 25            ' |               DI  (to SD)
  _SDpin_CLK    = 26            ' |               CLK (to SD)
  _SDpin_DO     = 24            ' |               DO  (fm SD)
' _SDpin_CD     = -1            ' | card detect   (can be DI)
' _SDpin_WP     = -1            ' / write protect (can be CLK) (not on microSD cards)
  _SDpins       = _SDpin_CS << 24 | _SDpin_DI << 16 | _SDpin_CLK << 8 | _SDpin_DO
'' +--------------------------------------------------------------------------+
#endif

  _LF_ON                        = 0                     ' 0= <lf> stripped
  
  _nullCharacter                = 0
  _homeCursorCharacter          = 1
  _backspaceCharacter           = 8
  _horizontalTabCharacter       = 9
  _lineFeedCharacter            = 10                                                                               
  _carriageReturnCharacter      = 13
  _clearScreenCharacter         = 16
  _quoteCharacter               = 34
  _backspaceKeycode             = 200

OBJ
  str   : "_StringE"
  fat   : "SD-MMC_FATEngine"
  fdx   : "FullDuplexSerial"

PUB main | errorString, cog

#ifdef delay1
  waitcnt(cnt + clkfreq*1)                              ' delay
#elseifdef delay5
  waitcnt(cnt + clkfreq*5)                              ' delay
#endif


  fdx.start(_SIpin, _SOpin, _SIOmode, _SIObaud)         ' start serial driver to PC

  crlf
  printString(string("_BOOT_OS v1.12:"))

#ifdef custom
  printString(string(" cust"))
#elseifdef 5M0
  printString(string(" 05M0"))
#elseifdef 10M0
  printString(string(" 10M0"))
#elseifdef 12M12
  printString(string(" P8X2"))
#elseifdef 6M5RB1
  printString(string(" RB1 "))
#endif

  printChar(",")
' "fatEngineStart" is only called once. Either driver object can call it.
'                          (  DO,        CLK,        DI,        CS,       WP, CD, _RTC_DAT, _RTC_CLK, -1)
  cog := fat.FATEngineStart( _SDpin_DO, _SDpin_CLK, _SDpin_DI, _SDpin_CS, -1, -1, -1,       -1,       -1)


  printString(string(" SD mount "))
  errorString := \fat.mountPartition(0)
  checkError(string("Mount Partition"), fat.partitionError, errorString) ' if error, display indefinately!!

  printString(fat.partitionFileSystemType)
  printChar(_quoteCharacter)                                        
  printString(fat.partitionVolumeLabel)
  printChar(_quoteCharacter)
    
  printString(string(", boot "))
  printString(@bootfile1)
  errorString := \fat.bootPartition(@bootfile1)         ' load/run <bootfile>
  if fat.partitionError <> $0D
    checkError(@bootfile1, fat.partitionError, errorString) ' if error, display indefinately!!

  printString(string(", boot "))
  printString(@bootfile2)
  errorString := \fat.bootPartition(@bootfile2)         ' load/run <bootfile>
  checkError(@bootfile2, fat.partitionError, errorString) ' if error, display indefinately!!

  repeat                                                ' how did we get here???
    printString(string("** OOPS! "))
    repeat 40
      printChar("!")
      waitcnt(cnt + clkfreq*2)
    crlf

PRI checkError(errorMessage, errorNumber, errorString)

  if errorNumber == 0
    return 0

  repeat                                                ' repeat indefinately !!!
    crlf
    printString(string("** Error ** "))
    printString(errorMessage)
    printString(String(" - "))
    printString(str.integerToHexadecimal(errorNumber,2))
    printString(string(": "))
    printStringCR(errorString)
    repeat 40
      printChar("?")
      waitcnt(cnt + clkfreq*2)
    
PRI PrintStringCR(pstring)      ' sends string to output devices (vga/serial) with CRLF
    PrintString(pstring)                            
    crlf

PRI PrintString(pstring)        ' sends string to output devices (vga/serial) with no CRLF
    repeat strsize(pstring)
      PrintChar(byte[pstring++])
     
PRI PrintChar(c)                ' sends character to output device
    if c <> _lineFeedCharacter
      fdx.tx(c)                                         ' send char  to output device
    elseif _LF_ON                                    
      fdx.tx(c)                                         ' send <lf>  to output device

PRI CRLF
    PrintChar(_carriageReturnCharacter)                 ' cr
    PrintChar(_lineFeedCharacter)                       ' lf (optional)


dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}